#    LLAMMA is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    LLAMMA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

#   company/organization: MR_ROBOT_COMPUTERS_REPAIR
#   Country: Italy
#   City: Barga (LU)
#   website: mrrobotcomputers.hopto.org
#   mail address: mrrobotcomputersrepair@gmail.com



package LLAMMA;
use XML::LibXML; # dipendenza
use feature "say";
use Try::Tiny;

use constant
{
	url => "http://www.lamma.rete.toscana.it/previ/ita/xml/comuni_web/dati/",
	_version => "LLAMMA v0.1",

	DAY => "giorno",
	MORNING => "mattina",
	AFTERNOON => "pomeriggio",
	EVENING => "sera",
	NIGHT => "notte",

	TEMP_MIN => "min",
	TEMP_MAX => "max",
	TEMP_REAL => "",
	TEMP_PERC => "perc",

	HYDRO_RISK => "idraulico",
	GEO_HYDRO_RISK => "idrogeologico",
	THUNDERSTORM_RISK => "temporali",
	WIND_RISK => "vento",
	SNOW_RISK => "neve",
	ICE_RISK => "ghiaccio",
	
};

@city_list;
$name="";
$your_city="";
@previsions;

sub getCityList # ok
{
	my $cityListUrl="http://www.lamma.rete.toscana.it/previ/ita/xml/lista_comuni.xml";
	my $xml=XML::LibXML->load_xml(location => $cityListUrl);

	foreach my $city ($xml->findnodes('pages/link/title'))
	{
		push(@city_list,$city->to_literal());
	}

	return @city_list;
}

sub selectCity # ok
{
	my $city=$_[1];
	my $_name=$_[1];

	$city=~ s/\s//g;
	$city=lc($city);
	try{
		my $cityURL="http://www.lamma.rete.toscana.it/previ/ita/xml/comuni_web/dati/$city.xml";
		my $xml=XML::LibXML->load_xml(location => $cityURL);
		$your_city=$xml;
		return $your_city;
	}
	catch{ return 0;}
}


1;

sub name #ok
{
	try{
		foreach my $name ($your_city->findnodes('dati/comune'))
		{
			return $name->to_literal();
		}
	} catch {return 0;}	
}

sub getUnixTimestamp #ok
{
	try{
		foreach my $timestamp ($your_city->findnodes('dati/time_ms'))
		{
			return $timestamp->to_literal();
		}
	}
	catch {return 0;}
}

sub lastUpdate #ok
{
	try{
		foreach my $update ($your_city->findnodes('dati/aggiornamento'))
		{
			return $update->to_literal();
		}
	}
	catch {return 0;}
}

sub sunrise #ok
{
	try{
		foreach my $sunrise ($your_city->findnodes('dati/almanacco/sole_sorge'))
		{
			return $sunrise->to_literal();
		}
	}
	catch {return 0;}
}

sub sunset  #ok
{
	try{
		foreach my $sunset ($your_city->findnodes('dati/almanacco/sole_tramonta'))
		{
			return $sunset->to_literal();
		}
	}
	catch {return 0;}
}

sub sun_height #ok
{
	try{
		foreach my $sun_height ($your_city->findnodes('dati/almanacco/sole_altezza'))
		{
			return $sun_height->to_literal();
		}
	}
	catch {return 0;}
}



sub max_sun_hour #ok
{
	try{
		foreach my $max_sun_hour ($your_city->findnodes('dati/almanacco/ora_altezza'))
		{
			return $max_sun_hour->to_literal();
		}
	}
	catch {return 0;}
}

sub moon_rises #ok
{
	try{
		foreach my $luna_sorge ($your_city->findnodes('dati/almanacco/luna_sorge'))
		{
			return $luna_sorge->to_literal();
		}
	}
	catch {return 0;}
}

sub moon_sets #ok
{
	try{
		foreach my $luna_tramonta ($your_city->findnodes('dati/almanacco/luna_tramonta'))
		{
			return $luna_tramonta->to_literal();
		}
	}
	catch {return 0;}
}

sub getPrevisions #ok
{
	try{
		foreach my $prev ($your_city->findnodes('dati/previsione'))
		{
			push(@previsions,$prev)
		}
	
		return @previsions;
	}
	catch {return 0;}
}

sub getDayByIdday #ok
{
	foreach my $day ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$day->attributes;

		if($_[1] == $attrs[0]->to_literal())
		{
			return $attrs[2]->to_literal();
		}
	}
}

sub getWeather #ok
{

	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;
		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @weather=$prev->getElementsByTagName('simbolo');
			if($weather[0])
			{
				@attrs=$weather[0]->attributes;
				return $attrs[0]->to_literal();
			}else {return undef;}
		}
	}
}

sub getWeatherStat_wind #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @weather=$prev->getElementsByTagName('simbolo');
			if($weather[1]){
				@attrs=$weather[1]->attributes;
				return $attrs[0]->to_literal();
			}else {return undef;}
		}
	}
}

sub getWeatherStat_temps #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @weather=$prev->getElementsByTagName('simbolo');
			if($weather[2])
			{
				@attrs=$weather[2]->attributes;
				return $attrs[0]->to_literal();
			}else {return undef;}
		}
	}
}

sub getUV #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			return $prev->getElementsByTagName('uv')->to_literal();
		}
	}
}

sub getTemps #ok
{
	if($_[2] eq DAY && $_[3] eq TEMP_REAL || $_[2] eq DAY && $_[3] eq TEMP_PERC){return "WARNING: 'DAY' + 'TEMP_REAL'/'TEMP_PERC' is not valid data schema: use 'TEMP_MIN' or 'TEMP_MAX' with  'DAY' constant";}
	if($_[2] eq MORNING && $_[3] eq TEMP_MIN || $_[2] eq MORNING && $_[3] eq TEMP_MAX){return "WARNING: 'MORNING' + 'TEMP_MIN'/'TEMP_MAX' is not valid data schema: use 'TEMP_REAL' or 'TEMP_PERC' with 'MORNING' constant";}
	if($_[2] eq AFTERNOON && $_[3] eq TEMP_MIN || $_[2] eq AFTERNOON && $_[3] eq TEMP_MAX){return "WARNING: 'AFTERNOON' + 'TEMP_MIN'/'TEMP_MAX' is not valid data schema: use 'TEMP_REAL' or 'TEMP_PERC' with 'AFTERNOON' constant";}
	if($_[2] eq EVENING && $_[3] eq TEMP_MIN || $_[2] eq EVENING && $_[3] eq TEMP_MAX){return "WARNING: 'EVENING' + 'TEMP_MIN'/'TEMP_MAX' is not valid data schema: use 'TEMP_REAL' or 'TEMP_PERC' with 'EVENING' constant";}
	if($_[2] eq NIGHT && $_[3] eq TEMP_MIN || $_[2] eq NIGHT && $_[3] eq TEMP_MAX){return "WARNING: 'NIGHT' + 'TEMP_MIN'/'TEMP_MAX' is not valid data schema: use 'TEMP_REAL' or 'TEMP_PERC' with 'NIGHT' constant";}

	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			if ($_[3] eq TEMP_MIN)
			{
				my @temps=$prev->getElementsByTagName('temp');
				return $temps[0]->to_literal();
			}

			if($_[3] eq TEMP_MAX)
			{
				my @temps=$prev->getElementsByTagName('temp');
				return $temps[1]->to_literal();
			}

			if($_[3] eq TEMP_REAL)
			{
				my @temps=$prev->getElementsByTagName('temp');
				return $temps[0]->to_literal;
			}

			if($_[3] eq TEMP_PERC)
			{
				my @temps=$prev->getElementsByTagName('temp');
				return $temps[1]->to_literal();
			}
		}
	}
}

sub getInversion #ok
{
	foreach $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;
		
		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @inversion=$prev->getElementsByTagName('inversione');
			return $inversion[0]->to_literal();
		}
	}
}

sub getHumidity #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @humidity=$prev->getElementsByTagName('um');
			return $humidity[0]->to_literal();
		}
	}
}

sub getRainProbability #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @rain_prob=$prev->getElementsByTagName('prob_rain');
			return @rain_prob[0]->to_literal();
		}
	}

}

sub getSnowLevel #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $_[2] eq $attrs[1]->to_literal())
		{
			my @snow_lv=$prev->getElementsByTagName('quota_neve');
			return $snow_lv[0]->to_literal();
		}
	}
}


sub getAlert #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $attrs[1]->to_literal() eq DAY)
		{
			my @alert=$prev->getElementsByTagName('allerta');
			if($alert[0]){
				my @alert_attrs=$alert[0]->attributes;
				return $alert_attrs[0]->to_literal();
			}else {return undef;}
		}
	}
}

sub getRisk #ok
{
	foreach my $prev ($your_city->findnodes('dati/previsione'))
	{
		@attrs=$prev->attributes;

		if($_[1] == $attrs[0]->to_literal() && $attrs[1]->to_literal() eq DAY)
		{
			if($_[2] eq HYDRO_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[0]->attributes;
					my $hydro_risk=$risk_attrs[1];
					return $hydro_risk->to_literal();
				} else {return undef;}
			}

			if($_[2] eq GEO_HYDRO_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[1]->attributes;
					my $geohydro_risk=$risk_attrs[1];
					return $geohydro_risk->to_literal();
				} else {return undef;}
			}

			if($_[2] eq THUNDERSTORM_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[2]->attributes;
					my $thunderstorm_risk=$risk_attrs[1];
					return $thunderstorm_risk->to_literal();
				} else {return undef;}
			}

			if($_[2] eq WIND_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[3]->attributes;
					my $wind_risk=$risk_attrs[1];
					return $wind_risk->to_literal();
				} else {return undef;}
			}

			if($_[2] eq SNOW_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[4]->attributes;
					my $snow_risk=$risk_attrs[1];
					return $snow_risk->to_literal();
				} else {return undef;}
			}

			if($_[2] eq ICE_RISK)
			{
				my @risk=$prev->getElementsByTagName('rischio');
				if($risk[0]){
					my @risk_attrs=$risk[5]->attributes;
					my $ice_risk=$risk_attrs[1];
					return $ice_risk->to_literal();
				} else {return undef;}
			}



		}
	}
}
